@@ -125,6 +125,8 @@ static const char *str_howto_type(const struct ksplice_reloc_howto *howto)
 		return "bug";
 	case KSPLICE_HOWTO_EXTABLE:
 		return "extable";
+	case KSPLICE_HOWTO_SYMBOL:
+		return "symbol";
 	default:
 		return "unknown";
 	}
@@ -172,6 +174,8 @@ void show_ksplice_section_flags(const struct ksplice_section *ksect)
 		printf(" text");
 	if (ksect->flags & KSPLICE_SECTION_DATA)
 		printf(" data");
+	if (ksect->flags & KSPLICE_SECTION_MATCH_DATA_EARLY)
+		printf(" match_early");
 	printf("\n");
 }
 
@@ -198,16 +202,12 @@ void show_ksplice_sections(struct supersect *ksect_ss)
 const char *str_ksplice_patch_type(struct supersect *ss,
 				   const struct ksplice_patch *kpatch)
 {
-	const unsigned short *line;
 	const char *const *strp;
 	struct supersect *data_ss;
 	switch(kpatch->type) {
 	case KSPLICE_PATCH_TEXT:
 		return strprintf("text\n  repladdr: %s", str_pointer
 				 (ss, (void *const *)&kpatch->repladdr));
-	case KSPLICE_PATCH_BUGLINE:
-		line = read_pointer(ss, &kpatch->contents, NULL);
-		return strprintf("bugline\n  line: %hx", *line);
 	case KSPLICE_PATCH_DATA:
 		return strprintf("data\n  size: %x", kpatch->size);
 	case KSPLICE_PATCH_EXPORT:
@@ -326,6 +326,12 @@ static void load_ksplice_reloc_offsets(struct superbfd *sbfd)
 		for (kreloc = ss->contents.data;
 		     (void *)kreloc < ss->contents.data + ss->contents.size;
 		     kreloc++) {
+			const struct ksplice_reloc_howto *khowto =
+			    read_pointer(ss, (void *const *)&kreloc->howto,
+					 NULL);
+			if (khowto->size == 0)
+				continue;
+
 			struct supersect *sym_ss;
 			const void *ptr =
 			    read_pointer(ss, (void *const *)&kreloc->blank_addr,
