@@ -37,6 +37,10 @@
 /* 8256e47cdc8923e9959eb1d7f95d80da538add80 was after 2.6.23 */
 #include <linux/marker.h>
 #endif /* LINUX_VERSION_CODE */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28)
+/* 97e1c18e8d17bd87e1e383b2e9d9fc740332c8e2 was after 2.6.27 */
+#include <linux/tracepoint.h>
+#endif /* LINUX_VERSION_CODE */
 
 #include <asm/uaccess.h>
 #include "offsets.h"
@@ -86,8 +90,14 @@ const struct table_section table_sections[]
 		.entry_contents_size = FIELD_ENDOF(struct bug_entry, flags),
 		.entry_align = __alignof__(struct bug_entry),
 		.has_addr = 1,
+#ifdef CONFIG_GENERIC_BUG_RELATIVE_POINTERS
+		.relative_addr = 1,
+		.addr_offset = offsetof(struct bug_entry, bug_addr_disp),
+		.relative_other = 1,
+		.other_offset = offsetof(struct bug_entry, file_disp),
+#else
 		.addr_offset = offsetof(struct bug_entry, bug_addr),
-		.other_offset = offsetof(struct bug_entry, line),
+#endif
 	},
 #else /* !CONFIG_GENERIC_BUG || LINUX_VERSION_CODE < */
 /* 91768d6c2bad0d2766a166f13f2f57e197de3458 was after 2.6.19 */
@@ -111,6 +121,16 @@ const struct table_section table_sections[]
 		.other_offset = offsetof(struct marker, name),
 	},
 #endif /* LINUX_VERSION_CODE */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28)
+/* 97e1c18e8d17bd87e1e383b2e9d9fc740332c8e2 was after 2.6.27 */
+	{
+		.sect = "__tracepoints",
+		.entry_size = sizeof(struct tracepoint),
+		.entry_align = __alignof__(struct tracepoint),
+		.other_sect = "__tracepoints_strings",
+		.other_offset = offsetof(struct tracepoint, name),
+	},
+#endif /* LINUX_VERSION_CODE */
 #ifdef CONFIG_PARAVIRT
 	{
 		.sect = ".parainstructions",
