@@ -24,6 +24,7 @@
  * Instead, we need to prevent Ksplice from patching thread_return.
  */
 extern const char thread_return[];
+EXTRACT_SYMBOL(thread_return);
 #define KSPLICE_IP(x) ((unsigned long)thread_return)
 #define KSPLICE_SP(x) ((x)->thread.rsp)
 #else /* Old x86 32-bit */
@@ -38,10 +39,31 @@ extern const char thread_return[];
 #include <linux/ftrace.h>
 
 extern ftrace_func_t ftrace_trace_function;
+EXTRACT_SYMBOL(ftrace_trace_function);
 #endif /* CONFIG_FTRACE */
 
 #define N_BITS(n) ((n) < sizeof(long) * 8 ? ~(~0L << (n)) : ~0L)
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
+/* 91768d6c2bad0d2766a166f13f2f57e197de3458 was after 2.6.19 */
+#if defined(_I386_BUG_H) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11) || \
+			     defined(CONFIG_DEBUG_BUGVERBOSE)) && \
+    !defined(do_each_thread_ve) /* OpenVZ */
+/* 38326f786cf4529a86b1ccde3aa17f4fa7e8472a was after 2.6.10 */
+		/* ud2 means BUG().  On old i386 kernels, it is followed
+		   by 2 bytes and then a 4-byte relocation; and is not
+		   disassembler-friendly. */
+		struct bug_frame {
+			unsigned char ud2[2];
+			unsigned short line;
+			char *filename;
+		} __attribute__((packed));
+#define KSPLICE_USE_BUG_FRAME
+#elif defined(__ASM_X8664_BUG_H)
+#define KSPLICE_USE_BUG_FRAME
+#endif
+#endif /* LINUX_VERSION_CODE */
+
 static abort_t compare_instructions(struct ksplice_mod_change *change,
 				    struct ksplice_section *sect,
 				    const struct ksplice_reloc **fingerp,
@@ -61,9 +83,9 @@ static uint8_t ud_operand_len(struct ud_operand *operand);
 static uint8_t ud_prefix_len(struct ud *ud);
 static long ud_operand_lval(struct ud_operand *operand);
 static int next_run_byte(struct ud *ud);
-static bool is_nop(struct ud *ud);
+static bool is_nop(struct ud *ud, const unsigned char *addr);
 static bool is_unconditional_jump(struct ud *ud);
-static bool is_mcount_call(struct ud *ud, unsigned long addr);
+static bool is_mcount_call(struct ud *ud, const unsigned char *addr);
 static void initialize_ksplice_ud(struct ud *ud);
 
 static abort_t arch_run_pre_cmp(struct ksplice_mod_change *change,
@@ -74,7 +96,9 @@ static abort_t arch_run_pre_cmp(struct ksplice_mod_change *change,
 {
 	abort_t ret;
 	const unsigned char *run, *pre, *run_start, *pre_start, *safety_start;
-	struct ud pre_ud, run_ud;
+	/* struct ud is big so we avoid putting it on the stack.  This
+	 * is safe because we are holding module_mutex. */
+	static struct ud pre_ud, run_ud;
 	const unsigned char **match_map;
 	const struct ksplice_reloc *finger;
 	unsigned long pre_offset, run_offset;
@@ -106,11 +130,19 @@ static abort_t arch_run_pre_cmp(struct ksplice_mod_change *change,
 	memset(match_map, 0, sizeof(*match_map) * sect->size);
 	match_map[0] = run_start;
 	sect->match_map = match_map;
+	sect->unmatched = 1;
 
 	while (1) {
 		if (pre_nop && ud_disassemble(&pre_ud) == 0) {
 			/* Ran out of pre bytes to match; we're done! */
 			unsigned long safety_offset = run - safety_start;
+			if (sect->unmatched != 0) {
+				if (mode == RUN_PRE_DEBUG)
+					ksdebug(change, "%d unmatched jumps\n",
+						sect->unmatched);
+				ret = NO_MATCH;
+				goto out;
+			}
 			ret = create_safety_record(change, sect, safety_records,
 						   (unsigned long)safety_start,
 						   safety_offset);
@@ -120,10 +152,8 @@ static abort_t arch_run_pre_cmp(struct ksplice_mod_change *change,
 			ret = NO_MATCH;
 			goto out;
 		}
-		pre_nop = is_nop(&pre_ud) ||
-		    is_mcount_call(&pre_ud, (unsigned long)pre);
-		run_nop = is_nop(&run_ud) ||
-		    is_mcount_call(&run_ud, (unsigned long)run);
+		pre_nop = is_nop(&pre_ud, pre) || is_mcount_call(&pre_ud, pre);
+		run_nop = is_nop(&run_ud, run) || is_mcount_call(&run_ud, run);
 		if (pre_nop && !run_nop) {
 			if (mode == RUN_PRE_DEBUG) {
 				ksdebug(change, "| nop: ");
@@ -164,7 +194,9 @@ static abort_t arch_run_pre_cmp(struct ksplice_mod_change *change,
 
 		if (match_map[pre_offset] == NULL) {
 			match_map[pre_offset] = run;
-		} else if (match_map[pre_offset] != run) {
+		} else if (match_map[pre_offset] == run) {
+			sect->unmatched--;
+		} else {
 			/* There is a discontinuity in the match map.
 			   Check that the last instruction was an
 			   unconditional change of control */
@@ -201,26 +233,26 @@ static abort_t arch_run_pre_cmp(struct ksplice_mod_change *change,
 				ret = NO_MATCH;
 				goto out;
 			}
+
+			sect->unmatched--;
 		}
 		run_offset = run - run_start;
 		run_unconditional = is_unconditional_jump(&run_ud);
 		run_nop = true;
 		pre_nop = true;
 
-#ifndef VZVERSION		/* OpenVZ */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20) && \
-    defined(_I386_BUG_H) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11) || \
-			     defined(CONFIG_DEBUG_BUGVERBOSE))
+    defined(KSPLICE_USE_BUG_FRAME)
 /* 91768d6c2bad0d2766a166f13f2f57e197de3458 was after 2.6.19 */
-/* 38326f786cf4529a86b1ccde3aa17f4fa7e8472a was after 2.6.10 */
 		if (run_ud.mnemonic == pre_ud.mnemonic &&
 		    run_ud.mnemonic == UD_Iud2) {
+			const struct bug_frame
+			    *pre_bug = (const struct bug_frame *)pre,
+			    *run_bug = (const struct bug_frame *)run;
 			const struct ksplice_reloc *r;
-			/* ud2 means BUG().  On old i386 kernels, it is followed
-			   by 2 bytes and then a 4-byte relocation; and is not
-			   disassembler-friendly. */
 			ret = lookup_reloc(change, &finger,
-					   (unsigned long)(pre + 4), &r);
+					   (unsigned long)&pre_bug->filename,
+					   &r);
 			if (ret == NO_MATCH) {
 				if (mode == RUN_PRE_INITIAL)
 					ksdebug(change, "Unrecognized ud2\n");
@@ -229,23 +261,30 @@ static abort_t arch_run_pre_cmp(struct ksplice_mod_change *change,
 			if (ret != OK)
 				goto out;
 			ret = handle_reloc(change, sect, r,
-					   (unsigned long)(run + 4), mode);
+					   (unsigned long)&run_bug->filename,
+					   mode);
 			if (ret != OK)
 				goto out;
 			/* If there's a relocation, then it's a BUG? */
 			if (mode == RUN_PRE_DEBUG) {
 				ksdebug(change, "[BUG?: ");
-				print_bytes(change, run + 2, 6, pre + 2, 6);
+				print_bytes(change,
+					    run + sizeof(run_bug->ud2),
+					    sizeof(*run_bug),
+					    pre + sizeof(pre_bug->ud2),
+					    sizeof(*pre_bug));
 				ksdebug(change, "] ");
 			}
-			pre += 8;
-			run += 8;
-			ud_input_skip(&run_ud, 6);
-			ud_input_skip(&pre_ud, 6);
+			pre += sizeof(*pre_bug);
+			run += sizeof(*run_bug);
+			ud_input_skip(&run_ud,
+				      sizeof(*run_bug) - sizeof(run_bug->ud2));
+			ud_input_skip(&pre_ud,
+				      sizeof(*pre_bug) - sizeof(pre_bug->ud2));
 			continue;
 		}
-#endif /* LINUX_VERSION_CODE && _I386_BUG_H && CONFIG_DEBUG_BUGVERBOSE */
-#endif /* VZVERSION */
+#endif /* LINUX_VERSION_CODE && KSPLICE_USE_BUG_FRAME */
+
 #ifdef CONFIG_XEN
 		if (run_ud.mnemonic == pre_ud.mnemonic &&
 		    run_ud.mnemonic == UD_Iud2) {
@@ -290,6 +329,7 @@ static abort_t compare_instructions(struct ksplice_mod_change *change,
 {
 	abort_t ret;
 	int i;
+	bool found_bug_entry = false;
 	const unsigned char *pre_start = (const unsigned char *)sect->address;
 	unsigned long pre_offset = pre - pre_start;
 	const struct ksplice_reloc *r;
@@ -314,30 +354,50 @@ static abort_t compare_instructions(struct ksplice_mod_change *change,
 		return UNEXPECTED;
 	}
 
-	ret = lookup_reloc(change, fingerp, (unsigned long)pre, &r);
-	if (ret == OK && (r->howto->type == KSPLICE_HOWTO_EXTABLE ||
-			  r->howto->type == KSPLICE_HOWTO_BUG)) {
+	while (1) {
+		ret = lookup_reloc(change, fingerp, (unsigned long)pre, &r);
+		if (ret == NO_MATCH)
+			break;
+		else if (ret != OK)
+			return ret;
+		else if (r->howto->size != 0)
+			break;
+
+		if (r->howto->type == KSPLICE_HOWTO_BUG)
+			found_bug_entry = true;
+
 		if (mode == RUN_PRE_DEBUG) {
 			if (r->howto->type == KSPLICE_HOWTO_EXTABLE)
 				ksdebug(change, "[ex] ");
 			if (r->howto->type == KSPLICE_HOWTO_BUG)
 				ksdebug(change, "[bug] ");
+			if (r->howto->type == KSPLICE_HOWTO_SYMBOL)
+				ksdebug(change, "[sym] ");
 		}
 		ret = handle_reloc(change, sect, r, (unsigned long)run, mode);
 		if (ret != OK)
 			return ret;
-	} else if (ret != NO_MATCH && ret != OK) {
-		return ret;
+		(*fingerp)++;
+	}
+
 #if defined(CONFIG_X86_64) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
 /* 91768d6c2bad0d2766a166f13f2f57e197de3458 was after 2.6.19 */
 #else /* !CONFIG_X86_64 || LINUX_VERSION_CODE >= */
-	} else if (run_ud->mnemonic == UD_Iud2) {
-#ifndef VZVERSION		/* OpenVZ */
-		ksdebug(change, "Unexpected ud2\n");
-		return NO_MATCH;
-#endif /* VZVERSION */
-#endif /* CONFIG_X86_64 && LINUX_VERSION_CODE */
+#ifndef do_each_thread_ve		/* OpenVZ */
+	if (run_ud->mnemonic == UD_Iud2 && !found_bug_entry) {
+		if (strcmp(change->target_name, "kvm_intel") == 0 ||
+		    strcmp(change->target_name, "kvm_amd") == 0) {
+			/* KVM has ud2a bugs without a bug table entry. */
+			if (mode == RUN_PRE_DEBUG)
+				ksdebug(change, "[kvm ud2]");
+		} else {
+			ksdebug(change, "Unexpected ud2\n");
+			return NO_MATCH;
+		}
 	}
+#endif /* do_each_thread_ve */
+#endif /* CONFIG_X86_64 && LINUX_VERSION_CODE */
+
 	for (i = 0; i < ARRAY_SIZE(run_ud->operand); i++) {
 		ret = compare_operands(change, sect, fingerp, run_start, run,
 				       pre, run_ud, pre_ud, i, mode);
@@ -455,11 +515,8 @@ static abort_t compare_operands(struct ksplice_mod_change *change,
 		    ud_operand_lval(pre_op);
 		const unsigned char *run_target = run + ud_insn_len(run_ud) +
 		    ud_operand_lval(run_op);
-		if (pre_target == run_target) {
-			/* Paravirt-inserted pcrel jump; OK! */
-			return OK;
-		} else if (pre_target >= pre_start &&
-			   pre_target < pre_start + sect->size) {
+		if (pre_target >= pre_start &&
+		    pre_target < pre_start + sect->size) {
 			/* Jump within the current function.
 			   Check it's to a corresponding place */
 			unsigned long new_pre_offset = pre_target - pre_start;
@@ -476,8 +533,12 @@ static abort_t compare_operands(struct ksplice_mod_change *change,
 				return NO_MATCH;
 			} else if (match_map[pre_target - pre_start] == NULL) {
 				match_map[pre_target - pre_start] = run_target;
+				sect->unmatched++;
 			}
 			return OK;
+		} else if (pre_target == run_target) {
+			/* Paravirt-inserted pcrel jump; OK! */
+			return OK;
 		} else {
 			if (mode == RUN_PRE_DEBUG) {
 				ksdebug(change, "<--Different operands!\n");
@@ -512,40 +573,37 @@ static void initialize_ksplice_ud(struct ud *ud)
 }
 
 #ifdef CONFIG_FTRACE
-static bool is_mcount_call(struct ud *ud, unsigned long addr)
+static bool is_mcount_call(struct ud *ud, const unsigned char *addr)
 {
-	unsigned long target = addr + ud_insn_len(ud) +
-	    ud_operand_lval(&ud->operand[0]);
+	const void *target =
+	    addr + ud_insn_len(ud) + ud_operand_lval(&ud->operand[0]);
 	if (ud->mnemonic == UD_Icall &&
-	    (target == (unsigned long)mcount ||
-	     target == (unsigned long)ftrace_trace_function))
+	    (target == mcount || target == ftrace_trace_function))
 		return true;
 	return false;
 }
 #else /* !CONFIG_FTRACE */
-static bool is_mcount_call(struct ud *ud, unsigned long addr)
+static bool is_mcount_call(struct ud *ud, const unsigned char *addr)
 {
 	return false;
 }
 #endif /* CONFIG_FTRACE */
 
-static bool is_nop(struct ud *ud)
+static bool is_nop(struct ud *ud, const unsigned char *addr)
 {
 	switch (ud->mnemonic) {
 	case UD_Inop:
 		return true;
 	case UD_Imov:
-		return ud->operand[0].type == UD_OP_REG &&
-		    ud->operand[1].type == UD_OP_REG &&
-		    ud->operand[2].type == UD_NONE &&
-		    ud->operand[0].base == ud->operand[1].base;
 	case UD_Ixchg:
-		return ud->operand[0].type == UD_OP_REG &&
+		return ud->dis_mode == 32 &&
+		    ud->operand[0].type == UD_OP_REG &&
 		    ud->operand[1].type == UD_OP_REG &&
 		    ud->operand[2].type == UD_NONE &&
 		    ud->operand[0].base == ud->operand[1].base;
 	case UD_Ilea:
-		return ud->operand[0].type == UD_OP_REG &&
+		return ud->dis_mode == 32 &&
+		    ud->operand[0].type == UD_OP_REG &&
 		    ud->operand[1].type == UD_OP_MEM &&
 		    ((ud->operand[1].base == ud->operand[0].base &&
 		      ud->operand[1].index == UD_NONE) ||
@@ -554,6 +612,35 @@ static bool is_nop(struct ud *ud)
 		      ud->operand[1].scale == 0)) &&
 		    ud_operand_lval(&ud->operand[1]) == 0 &&
 		    ud->operand[2].type == UD_NONE;
+	case UD_Ijmp:
+		/* jmp +N followed by N 0x90s is a NOP */
+		if (ud->operand[0].type == UD_OP_JIMM &&
+		    ud->operand[1].type == UD_NONE &&
+		    ud->operand[2].type == UD_NONE &&
+		    ud_operand_len(&ud->operand[0]) == 1) {
+			/* struct ud is big so we avoid putting it on the stack.
+			 * This is safe because we are holding module_mutex. */
+			static struct ud temp_ud;
+			int len = ud_operand_lval(&ud->operand[0]);
+			int i;
+
+			if (len < 0 || len > 13)
+				return false;
+
+			initialize_ksplice_ud(&temp_ud);
+			ud_set_input_hook(&temp_ud, next_run_byte);
+			ud_set_user_opaque_data(&temp_ud,
+						(unsigned char *)addr +
+						ud_insn_len(ud));
+
+			for (i = 0; i < len; i++) {
+				if (ud_disassemble(&temp_ud) == 0)
+					return false;
+				if (temp_ud.mnemonic != UD_Inop)
+					return false;
+			}
+			return true;
+		}
 	default:
 		return false;
 	}
