@@ -179,6 +179,14 @@ DECLARE_VEC_TYPE(struct label_map, label_map_vec);
 DECLARE_HASH_TYPE(struct label_map *, label_mapp_hash, label_mapp_hash_init,
 		  label_mapp_hash_free, label_mapp_hash_lookup);
 
+struct entry_point {
+	const char *label;
+	const char *name;
+	long offset;
+	asymbol *symbol;
+};
+DECLARE_VEC_TYPE(struct entry_point, entry_point_vec);
+
 struct span {
 	struct supersect *ss;
 	asymbol *symbol;
@@ -194,6 +202,8 @@ struct span {
 	bool datapatch;
 	bool precallable;
 	struct span *match;
+	struct entry_point_vec entry_points;
+	struct entry_point_vec pre_entry_points;
 	bfd_size_type shift;
 };
 DECLARE_VEC_TYPE(struct span, span_vec);
@@ -212,7 +222,8 @@ struct superbfd {
 enum supersect_type {
 	SS_TYPE_TEXT, SS_TYPE_DATA, SS_TYPE_RODATA, SS_TYPE_STRING,
 	SS_TYPE_SPECIAL, SS_TYPE_IGNORED, SS_TYPE_KSPLICE, SS_TYPE_EXPORT,
-	SS_TYPE_EXIT, SS_TYPE_KSPLICE_CALL, SS_TYPE_UNKNOWN
+	SS_TYPE_EXIT, SS_TYPE_KSPLICE_CALL, SS_TYPE_KSPLICE_EXTRACT,
+	SS_TYPE_BUGTABLE, SS_TYPE_UNKNOWN
 };
 
 struct supersect {
@@ -232,6 +243,7 @@ struct supersect {
 	bool keep;
 	enum supersect_type type;
 	enum supersect_type orig_type;
+	bool match_data_early;
 };
 
 struct superbfd *fetch_superbfd(bfd *abfd);
@@ -255,7 +267,7 @@ void sect_do_copy(struct supersect *dest_ss, void *dest,
 	 strcmp(&str[strlen(str) - strlen(suffix)], suffix) == 0)
 
 bfd_vma addr_offset(struct supersect *ss, const void *addr);
-bfd_vma get_reloc_offset(struct supersect *ss, arelent *reloc, bool adjust_pc);
+bfd_vma reloc_offset(struct supersect *ss, arelent *reloc);
 arelent *find_reloc(struct supersect *ss, const void *addr);
 bfd_vma read_reloc(struct supersect *ss, const void *addr, size_t size,
 		   asymbol **symp);
